/* ---------------------------------------------------------------- */
//
// AppDelegate.m
// ADC
//
// Created by Christoph Birk on 2017-10-17
// Copyright (c) 2017 Christoph Birk. All rights reserved.
//
/* ---------------------------------------------------------------- */

#define DEBUG            1

#define RUN_SERVER
#define UNIT_TEST        0

/* ---------------------------------------------------------------- */

#import  "AppDelegate.h"

#import  "Yutils.h"
#import  "Notice.h"
#ifdef RUN_SERVER
#import  "Tcpip.h"
#endif

#include "main.h"
#include "utils.h"

/* ---------------------------------------------------------------- */

#define DBE_AUTOSTART  "dbe_autostart"

/* ---------------------------------------------------------------- */

char   *p_exec=NULL,*p_build=NULL,*p_version=NULL,*p_ident=NULL;
char   *r_path=NULL,*s_path=NULL;
Logger *main_logger=nil;
BOOL   use_shared=FALSE;

/* ---------------------------------------------------------------- */

@implementation AppDelegate
{
#ifdef RUN_SERVER
  TCPIP_Server *tcpipServer;           // b0048
#endif
}
@synthesize panel_calibrate;

/* ---------------------------------------------------------------- */
#pragma mark "Application Delegate"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
  char buf[128];
  NSString *str;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,aNotification);
#endif
  
  YbundleInfo(&p_exec,&p_build,&p_version,NULL,&r_path,&p_ident);
  // printf("%s\n",r_path);

  // defaults write <app domain name> NSAppSleepDisabled -bool YES
  if (!YappNapDisabled()) {
    [Notice openNotice:"Warning: AppNap enabled\rPlease restart the GUI"
            title:P_TITLE time:0 level:1];
  }
  // reset: defaults read edu.carnegiescience.obs.ADC

  NSArray *users = [NSArray arrayWithObjects:@"birk",nil];
  if ([users indexOfObject:NSUserName()] == NSNotFound) {
    use_shared = YprefSetupShared();
  } else {
    use_shared = NO;
  }
  (void)YappSupportCreate(&s_path,use_shared);

  main_logger = [[Logger alloc] initWithBase:nil useShared:use_shared period:0 useUT:YES];
  main_logger.mode = LOGGER_FLUSH;        // keep open, auto-flush
  str = [NSString stringWithFormat:@"user= %@ , sharedSetup= %d , UT= %s",
         NSUserName(),use_shared,get_ut_timestr(buf,cor_time(0))];
  [main_logger append:[str UTF8String]];  // added UT b0044
  [main_logger setAutoChange:YES];        // b0049

  
  [edit_acroA setStringValue:YprefGetString(@DBE_ACRO_A_HOST,@DEF_ACRO_A_HOST,use_shared)];
  [edit_acroD setStringValue:YprefGetString(@DBE_ACRO_D_HOST,@DEF_ACRO_D_HOST,use_shared)];
  
  [popup_acroA selectItemAtIndex:YprefGetInt(DBE_ACRO_A_ONLINE,DEF_STATE,use_shared)];
  [popup_acroD selectItemAtIndex:YprefGetInt(DBE_ACRO_D_ONLINE,DEF_STATE,use_shared)];
  
  [edit_tcs setStringValue:YprefGetString(@DBE_TCS_HOST,@DEF_TCS_HOST,use_shared)];
  [popup_tcs selectItemAtIndex:YprefGetInt(DBE_TCS_ONLINE,DEF_STATE,use_shared)];

  if (YprefGetInt(DBE_AUTOSTART,0,NO)) {
    [self action_startup:nil];
  }
}

/* ---------------------------------------------------------------- */

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication*)appl
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,appl);
#endif

  return YES;
}

/* ---------------------------------------------------------------- */

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication*)appl
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,appl);
#endif

  return NSTerminateNow;
}

/* ---------------------------------------------------------------- */

- (void)applicationWillTerminate:(NSNotification *)note
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,note);
#endif

#ifdef RUN_SERVER
  [tcpipServer stop];
#endif
  [main_logger append:PREFUN];
  [main_logger close];
}

/* ---------------------------------------------------------------- */
#pragma mark "Actions (Menu)"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (IBAction)action_startup:(id)sender
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif

  YprefPutString(@DBE_ACRO_A_HOST,[edit_acroA stringValue],use_shared);
  YprefPutString(@DBE_ACRO_D_HOST,[edit_acroD stringValue],use_shared);
  YprefPutInt(DBE_ACRO_A_ONLINE,(int)[popup_acroA indexOfSelectedItem],use_shared);
  YprefPutInt(DBE_ACRO_D_ONLINE,(int)[popup_acroD indexOfSelectedItem],use_shared);

  YprefPutString(@DBE_TCS_HOST,[edit_tcs stringValue],use_shared);
  YprefPutInt(DBE_TCS_ONLINE,(int)[popup_tcs indexOfSelectedItem],use_shared);

  [window_config orderOut:self];
    
  assert(adcController);
  [adcController.window makeKeyAndOrderFront:self];
  [adcController performSelectorInBackground:@selector(run_init) withObject:nil];
  
#ifdef RUN_SERVER
  int port = 50001+100*PROJECT_ID;
  tcpipServer = [[TCPIP_Server alloc] initWithPort:port delegate:self];
#ifdef NDEBUG
  tcpipServer.throttle = 0.5f;      // max. 2 connections per second
#endif
  dispatch_async(GLOBAL_QUEUE,^(void) { char buf[128];
    int err = [tcpipServer start];
    if (err) {
      sprintf(buf,"%s: TCP/IP server failed, err=%d",P_TITLE,err);
      [main_logger append:buf];
      YnotifyNotice(buf,"n/a",0,2);
      [NSApp terminate:self];
    }
    sprintf(buf,"listening to port %d\n",port);
    [main_logger append:buf];
#if (UNIT_TEST == 1)
    if (!err) { TCPIP_Connection *con; char res[128];
      Ysleep(2000);
      con = [[TCPIP_Connection alloc] initWithHost:"localhost" port:port online:YES];
      for (int i=0; i<3; i++) {
        [con request:"version\n" response:res max:sizeof(res)]; Ysleep(500);
      }
      [con release];
    }
#endif // UNIT TEST 1
  });
#endif
}

/* ---------------------------------------------------------------- */

- (IBAction)action_prefs:(id)sender
{
  char title[64];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif
  assert([NSThread isMainThread]);
  
  Ystr2chr([sender title],title,sizeof(title));
  if (strstr(title,"Preferences")) { // show Preferences
    [chk_autostart setState:(YprefGetInt(DBE_AUTOSTART,0,NO) ? NSOnState : NSOffState)];
    [edit_interval setIntValue:YprefGetInt(DBE_INTERVAL,DEF_INTERVAL,use_shared)];
    [popup_mode selectItemAtIndex:_prism_mode];
    [panel_prefs makeKeyAndOrderFront:self];
  } else
  if (strstr(title,"Apply")) {           // apply Preferences
    YprefPutInt(DBE_AUTOSTART,([chk_autostart state]==NSOnState) ? 1 : 0,NO);
    YprefPutInt(DBE_INTERVAL,imax(1,[edit_interval intValue]),use_shared);
    _prism_mode = (int)[popup_mode indexOfSelectedItem]; // b0037
    [panel_prefs orderOut:self];
    [self updateRunMode];
    [adcController reloadPreferences];
  } else {
    assert(0);
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_file:(id)sender
{
  char title[64];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif
  
  Ystr2chr([sender title],title,sizeof(title));
  
  if (strstr(title,"Logfile")) {    // show Logfile(s)
    [[NSWorkspace sharedWorkspace] openFile:main_logger.logfile];
  } else
  if (strstr(title,"Calibrate")) {  // show Calibration window
    [panel_calibrate makeKeyAndOrderFront:self];
  } else {
    assert(0);
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_help:(id)sender
{
  char title[64];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif

  Ystr2chr([sender title],title,sizeof(title));

  if (strstr(title,"Software")) {
    Ywww("http://instrumentation.obs.carnegiescience.edu/Software/ADC/");
  }
}

/* ---------------------------------------------------------------- */
#pragma mark "Others"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (void)goodbye:(NSString*)text error:(int)err
{
  YprefPutInt(DBE_AUTOSTART,0,NO);

  NSString *string = [NSString stringWithFormat:@"%@, err=%d",text,err];
  NSString *info = [ADC_Controller errorString:err];
  NSImage *image = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle]
                      pathForResource:@"macOSbomb" ofType:@"png"]];
  dispatch_async(MAIN_QUEUE,^(void) {
    NSImageView *view = [[NSImageView alloc]
                         initWithFrame:NSMakeRect(0,0,image.size.width,image.size.height)];
    [view setImage:image];
    NSAlert *alert = [[NSAlert alloc] init];
    if (text) [alert setMessageText:string];
    if (info) [alert setInformativeText:info];
    [alert addButtonWithTitle:@"ok"];
    [alert setAccessoryView:view];
    [view release]; 
    [alert runModal];
    [alert release];
    [NSApp terminate:nil];
  });
  [image release];
}

/* ---------------------------------------------------------------- */

- (void)updateRunMode                  // b0037
{
  NSImage *img=nil;
  NSString *title=@"Park";
#if (DEBUG > 1)
  fprintf(stderr,"%s: mainThread=%d\n",PREFUN,[NSThread isMainThread]);
#endif
  assert(adcController);
  
  BOOL eng_mode = (self.prism_mode != 0) ? TRUE : FALSE;
  if (adcController.adcState == 1) {   // ADC inserted
    if (!adcController.control_state) eng_mode = TRUE;
    if (!adcController.update_state) eng_mode = TRUE;
  }
  if (eng_mode) {
    NSString *file = [NSString stringWithFormat:@"%s/warning.png",r_path];
    img = [[NSImage alloc] initWithContentsOfFile:file];
  }
  
  if (!adcController.control_state) title = @"Run";
  if (!adcController.update_state) title = @"Run";
  if (!adcController.adcState) title = @"Run";

  dispatch_async(MAIN_QUEUE,^(void) {
    [view_eng setImage:img];
    [adcController.but_run setTitle:title]; // b0040
  });
  [img release];
}

/* ---------------------------------------------------------------- */
#pragma mark "TCPIP_Server delegate"
/* ---------------------------------------------------------------- */

#ifdef RUN_SERVER
- (char*)tcpip_handler:(const char*)command from:(NSString*)ipNumber socket:(int)msgsock
{
  char cmd[128]="",par1[128]="",par2[128]="";
  NSString *string = @(command);
  NSString *line = [string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%s,%s,%d)\n",PREFUN,[line UTF8String],[ipNumber UTF8String],msgsock);
#endif

  char *answer = (char*)malloc(512*sizeof(char));
  *answer = '\0';
  if (strlen(command) >= sizeof(cmd)) return answer;

  string = [NSString stringWithFormat:@"TCP: %@-> %@",ipNumber,line];
  [main_logger append:[string UTF8String]];
  sscanf([line UTF8String],"%s %s %s",cmd,par1,par2);
  
  if (!strcmp(cmd,"version")) {        // version check -----------
    sprintf(answer,"%s (%s)",p_version,p_build);
  } else
  if (!strcmp(cmd,"status")) {         // status b0048
    sprintf(answer,"%s %s",(adcController.adcState) ? "in" : "out",
                           adcController.control_state ? "auto" : "off");
  } else {                             // unknown command
    sprintf(answer,"unknown command '%s'",cmd);
    Ysleep(1000);                      // throttle
  }
  assert(answer);

  string = [NSString stringWithFormat:@"TCP: %@<- %s",ipNumber,answer];
  [main_logger append:[string UTF8String]];
  strcat(answer,"\n");                 // terminating <LF>
  
  return answer;
}
#endif

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
