/* ---------------------------------------------------------------- */
//
// PlotController.h
//
// Created by Christoph Birk on 2019-02-07
//
/* ---------------------------------------------------------------- */

#import <Cocoa/Cocoa.h>

/* ---------------------------------------------------------------- */

typedef struct { 
  float r,g,b;
} PlotColor;

/* ---------------------------------------------------------------- */

@interface PlotDataPoint : NSObject
{
}
@property (readwrite,nonatomic) double x,y;
+ (id)pointWithX:(double)x y:(double)y;
- (id)initWithX:(double)x y:(double)y;
@end

/* ---------------------------------------------------------------- */

enum plot_scale_enum { PLOT_SCALE_MAN,PLOT_SCALE_FULL }; 

@interface PlotView : NSView {
  NSArray    *dataArrays;
  PlotColor  *colors;
  NSString   *unit;
  int        scale_xmode,scale_ymode;
  double     scale_xmin,scale_xmax,scale_ymin,scale_ymax;
  CGFloat    font_size;
  CTFontRef  font;
}

@property (assign,nonatomic) int      scale_xmode,scale_ymode;
@property (assign,nonatomic) double   scale_xmin,scale_xmax;
@property (assign,nonatomic) double   scale_ymin,scale_ymax;
@property (copy,nonatomic)   NSString *unit;

- (void)setDataArray:(NSArray*)array colors:(const PlotColor*)cols;

@end

/* ---------------------------------------------------------------- */

@interface PlotController : NSWindowController {
  IBOutlet PlotView      *view;
  IBOutlet NSPanel       *panel_pars;
  IBOutlet NSPopUpButton *popup_xmode,*popup_ymode;
  IBOutlet NSTextField   *edit_xmin,*edit_xmax,*edit_ymin,*edit_ymax; 
  IBOutlet NSButton      *but_apply;
}

@property (readonly,nonatomic) PlotView *view;

- (id)initWithWindowNib:(NSString*)name context:(int)myid
                                        title:(const char*)tstr
                                        unit:(const char*)ustr
                                        N:(int)nc
                                        colors:(const PlotColor*)cols;

- (IBAction)action_pars:(id)sender;

- (void)addData:(int)c x:(double)x y:(double)y redraw:(BOOL)r;
- (void)clearData;
- (void)removeData:(int)n;

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
