/* ---------------------------------------------------------------- */
//
// ADC_Controller.h
// ADC
//
// Created by Christoph Birk on 2017-10-18
// Copyright (c) 2017 Christoph Birk. All rights reserved.
//
/* ---------------------------------------------------------------- */

@import Cocoa;

#include  "main.h"                   // Simulator only flag

/* ---------------------------------------------------------------- */

#define DBE_ACRO_A_HOST   "dbe_acromagA_host"
#define DBE_ACRO_D_HOST   "dbe_acromagD_host"

#ifdef SIM_ONLY
#define DEF_ACRO_A_HOST   "localhost"       // User
#define DEF_ACRO_D_HOST   "localhost"       // password00
#else
#define DEF_ACRO_A_HOST   "139.229.109.65"  // SBS: 10.7.1.169
#define DEF_ACRO_D_HOST   "139.229.109.66"  // SBS: 10.7.1.170
#endif
#define DBE_ACRO_A_ONLINE "dbe_acromagA_online"
#define DBE_ACRO_D_ONLINE "dbe_acromagD_online"

#define DBE_TCS_HOST      "dbe_tcs_host"
#ifdef NDEBUG
#define DEF_TCS_HOST      "mag2tcs"
#else
#define DEF_TCS_HOST      "andromeda"
#endif
#define DBE_TCS_ONLINE    "dbe_tcs_online"

#ifdef SIM_ONLY
#define DEF_STATE         0
#else
#define DEF_STATE         1
#endif

#define DBE_INTERVAL      "dbe_control_interval"
#define DEF_INTERVAL      30 

/* ---------------------------------------------------------------- */

@interface ADC_Controller : NSWindowController
{
  // GUI
  IBOutlet NSTextField   *edit_elevation,*edit_rotator;
  IBOutlet NSButton      *but_update,*chk_update;
  IBOutlet NSProgressIndicator *prog_adc,*prog_lensA,*prog_lensB;
  IBOutlet NSColorWell   *col_adcIn,*col_adcOut;
  IBOutlet NSPopUpButton *popup_adc,*popup_control;
  IBOutlet NSTextField   *edit_encA,*edit_targetA;
  IBOutlet NSTextField   *edit_encB,*edit_targetB;
  IBOutlet NSPopUpButton *popup_other; 
  IBOutlet NSProgressIndicator *prog_other;
  IBOutlet NSColorWell   *col_otherIn,*col_otherOut;
  IBOutlet NSButton      *but_run;
  // Calibration Window
  IBOutlet NSPopUpButton *pop_lens;
  IBOutlet NSButton      *but_calibrate,*but_plus,*but_minus,*but_stop;
  IBOutlet NSTextField   *edit_phase,*edit_angle;
  IBOutlet NSTextField   *edit_sin,*edit_min_sin,*edit_max_sin;
  IBOutlet NSTextField   *edit_cos,*edit_min_cos,*edit_max_cos;
}

@property (readonly,atomic) BOOL control_state,update_state;
@property (readonly,atomic) int  adcState,otherState; // other NEW b0058
@property (readonly,nonatomic) NSButton *but_run;

+ (NSString*)errorString:(int)err;

- (void)reloadPreferences;
- (void)run_init;

- (IBAction)action_telescope:(id)sender;
- (IBAction)action_slide:(id)sender;
- (IBAction)action_lens:(id)sender;
- (IBAction)action_control:(id)sender;
- (IBAction)action_calibrate:(id)sender;
- (IBAction)action_run:(id)sender; 

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
